from mock import patch, MagicMock
from nose.tools import assert_equal, assert_raises, assert_true

import datetime
import outbrain
import unittest
import yaml


class TestOutbrainAmplifyApi(unittest.TestCase):
    """
    Test suite for OutbrainAmplifyApi class
    """

    @patch('outbrain.OutbrainAmplifyApi.get_token')
    def test_init(self, get_token_mock):
        get_token_mock.return_value = 'token_mock'
        config = yaml.load(open('outbrain.yml.example', 'r'))
        api = outbrain.OutbrainAmplifyApi(outbrain_config=config)
        get_token_mock.assert_called_with('user', 'pass')
        assert_equal(api.token, "token_mock")

    @patch('requests.get')
    @patch('outbrain.OutbrainAmplifyApi.get_token', MagicMock(return_value="token_mock"))
    def test_request(self, requests_get_mock):
        requests_get_mock.return_value = create_mock_request_result()

        config = yaml.load(open('outbrain.yml.example', 'r'))
        api = outbrain.OutbrainAmplifyApi(outbrain_config=config)
        assert_equal(api.token, "token_mock")
        result = api._request('path_mock', 'params_mock')
        assert_true(isinstance(result, dict))
        assert_equal(result, {'request_key': 'request_value'})

    @patch('requests.get')
    def test_get_token(self, requests_get_mock):
        requests_get_mock.return_value = create_mock_token()

        config = yaml.load(open('outbrain.yml.example', 'r'))
        api = outbrain.OutbrainAmplifyApi(outbrain_config=config)
        assert_equal(api.token, {"mock_key": "mock_value"})

    @patch('outbrain.OutbrainAmplifyApi.get_token', MagicMock())
    def test_get_marketer(self):
        config = yaml.load(open('outbrain.yml.example', 'r'))
        api = outbrain.OutbrainAmplifyApi(outbrain_config=config)

        api._request = MagicMock(return_value='result_mock')
        assert_raises(TypeError, api.get_marketer) # needs an argument
        result = api.get_marketer('my_id')
        api._request.assert_called_with('marketers/my_id')
        assert_equal(result, 'result_mock')

    @patch('outbrain.OutbrainAmplifyApi.get_token', MagicMock())
    def test_get_all_marketer_ids(self):
        config = yaml.load(open('outbrain.yml.example', 'r'))
        api = outbrain.OutbrainAmplifyApi(outbrain_config=config)
        api.get_marketers = MagicMock(return_value=[{'id': 1}, {'id': 2}])
        res = api.get_marketer_ids()
        assert_equal(res, [1, 2])

    @patch('outbrain.OutbrainAmplifyApi.get_token', MagicMock())
    def test_get_budget(self):
        config = yaml.load(open('outbrain.yml.example', 'r'))
        api = outbrain.OutbrainAmplifyApi(outbrain_config=config)

        api._request = MagicMock(return_value='result_mock')
        assert_raises(TypeError, api.get_budget) # needs an argument
        result = api.get_budget('my_budget_id')
        api._request.assert_called_with('budgets/my_budget_id')
        assert_equal(result, 'result_mock')

    @patch('outbrain.OutbrainAmplifyApi.get_token', MagicMock())
    def test_get_budgets_per_marketer(self):
        config = yaml.load(open('outbrain.yml.example', 'r'))
        api = outbrain.OutbrainAmplifyApi(outbrain_config=config)

        api._request = MagicMock()
        marketer_ids = []
        api.get_budgets_per_marketer([])
        assert_equal(api._request.call_count, 0)

        api._request = MagicMock(return_value={'budgets': 'budget_mock'})
        result = api.get_budgets_per_marketer(['foo'])
        assert_equal(api._request.call_count, 1)
        api._request.assert_called_with('/marketers/foo/budgets')
        assert_equal(result, {'foo': 'budget_mock'})

        api._request = MagicMock()
        marketer_ids = [m for m in range(42)]
        api.get_budgets_per_marketer(marketer_ids)
        assert_equal(api._request.call_count, len(marketer_ids))

    @patch('outbrain.OutbrainAmplifyApi._request')
    @patch('outbrain.OutbrainAmplifyApi.get_token', MagicMock())
    def test_get_campaign(self, _request_mock):
        config = yaml.load(open('outbrain.yml.example', 'r'))
        api = outbrain.OutbrainAmplifyApi(outbrain_config=config)

        path = 'campaigns/campaign_id_mock'
        _request_mock.return_value = '_request_return_mock'
        result = api.get_campaign('campaign_id_mock')
        _request_mock.assert_called_with(path)
        assert_equal(result, _request_mock.return_value)

    @patch('outbrain.OutbrainAmplifyApi._request')
    @patch('outbrain.OutbrainAmplifyApi.get_token', MagicMock())
    def test_get_campaign_ids(self, _request_mock):
        config = yaml.load(open('outbrain.yml.example', 'r'))
        api = outbrain.OutbrainAmplifyApi(outbrain_config=config)

        api._yield_all_campaigns = MagicMock(return_value= [])
        res = api.get_campaign_ids()
        assert_equal(res, [])

        api._yield_all_campaigns = MagicMock(return_value= [{'id': 'x'}, {'id': 'y'}])
        res = api.get_campaign_ids()
        assert_equal(res, ['x', 'y'])

    @patch('outbrain.OutbrainAmplifyApi.get_token', MagicMock())
    def test_get_campaigns_per_budget(self):
        config = yaml.load(open('outbrain.yml.example', 'r'))
        api = outbrain.OutbrainAmplifyApi(outbrain_config=config)

        api._request = MagicMock()
        api.get_campaigns_per_budget([])
        assert_equal(api._request.call_count, 0)

        api._request = MagicMock(return_value={'campaigns': 'budget_mock'})
        result = api.get_campaigns_per_budget(['foo'])
        assert_equal(api._request.call_count, 1)
        api._request.assert_called_with('budgets/foo/campaigns')
        assert_equal(result, {'foo': 'budget_mock'})

        api._request = MagicMock()
        budget_ids = [b for b in range(7)]
        api.get_campaigns_per_budget(budget_ids)
        assert_equal(api._request.call_count, len(budget_ids))

    @patch('outbrain.OutbrainAmplifyApi._request')
    @patch('outbrain.OutbrainAmplifyApi.get_token', MagicMock())
    def test_get_campaigns_per_marketer(self, _request_mock):
        config = yaml.load(open('outbrain.yml.example', 'r'))
        api = outbrain.OutbrainAmplifyApi(outbrain_config=config)
        path = 'marketers/abc123/campaigns'

        assert_raises(TypeError, api.get_campaigns_per_marketer)

        _request_mock.reset_mock()
        api.get_campaigns_per_marketer([])
        assert not _request_mock.called

        _request_mock.reset_mock()
        api.get_campaigns_per_marketer(['abc123'])
        _request_mock.assert_called_with(path, {'includeArchived': 'true'})

        _request_mock.reset_mock()
        api.get_campaigns_per_marketer(['abc123'], include_archived=True)
        _request_mock.assert_called_with(path, {'includeArchived': 'true'})

        _request_mock.reset_mock()
        api.get_campaigns_per_marketer(['abc123'], include_archived=False)
        _request_mock.assert_called_with(path, {'includeArchived': 'false'})

    @patch('requests.get', MagicMock())
    @patch('outbrain.OutbrainAmplifyApi._yield_promoted_links_for_campaign',
           MagicMock(return_value=[1, 'b', 3]))
    @patch('outbrain.OutbrainAmplifyApi.get_token', MagicMock())
    def test_get_promoted_links_for_campaign(self):
        config = yaml.load(open('outbrain.yml.example', 'r'))
        api = outbrain.OutbrainAmplifyApi(outbrain_config=config)
        result = api.get_promoted_links_for_campaign('campaign_id_mock')
        assert_equal(result, [1, 'b', 3])

    @patch('outbrain.OutbrainAmplifyApi.get_token', MagicMock())
    def test_page_performance_data(self):
        config = yaml.load(open('outbrain.yml.example', 'r'))
        api = outbrain.OutbrainAmplifyApi(outbrain_config=config)

        start = datetime.datetime(year=2015, month=4, day=1, hour=13, minute=42)
        end = datetime.datetime(year=2015, month=4, day=3, hour=19, minute=24)
        path = 'marketers/marketer_id_mock/performanceByPublisher'
        params = {'to': '2015-04-01', 'from': '2015-04-03', 'limit': 20, 'offset': 3}

        path = 'marketers/marketer_id_mock/performanceByPublisher'
        params = {'from': '2015-04-01', 'to': '2015-04-03', 'limit': 20, 'offset': 3}
        api._request = MagicMock(return_value={})
        result = api._get_performance_data(path, start, end, 20, 3)
        api._request.assert_called_with(path, params)
        assert_equal(result, [])

        api._request = MagicMock(return_value={'details': 'details_mock'})
        result = api._get_performance_data(path, start, end, 20, 3)
        api._request.assert_called_with(path, params)
        assert_equal(result, 'details_mock')

    @patch('requests.get', MagicMock())
    @patch('outbrain.OutbrainAmplifyApi.get_token', MagicMock())
    @patch('outbrain.OutbrainAmplifyApi._request')
    def test_page_promoted_links_for_campaign(self, _request_mock):
        config = yaml.load(open('outbrain.yml.example', 'r'))
        api = outbrain.OutbrainAmplifyApi(outbrain_config=config)

        _request_mock.reset_mock()
        _request_mock.return_value = {'promotedLinks': []}
        path = 'campaigns/123abc/promotedLinks'
        res = api._page_promoted_links_for_campaign(path, True, [], 10, 0)
        params = {'enabled': 'true', 'limit': 10, 'offset': 0}
        _request_mock.assert_called_with(path, params)
        assert_equal(res, [])
        
        _request_mock.reset_mock()
        _request_mock.return_value = {'promotedLinks': [1, 2, 'c']}
        path = 'campaigns/a1b2c3/promotedLinks'
        res = api._page_promoted_links_for_campaign(path, False, ['APPROVED', 'PENDING'], 12, 2)
        params = {'enabled': 'false', 'limit': 12, 'offset': 2, 'statuses': 'APPROVED,PENDING'}
        _request_mock.assert_called_with(path, params)
        assert_equal(res, [1, 2, 'c'])

        _request_mock.return_value = {'foo': 2}
        path = 'campaigns/a1b2c3/promotedLinks'
        res = api._page_promoted_links_for_campaign(path, None, ['PENDING'], 15, 5)
        params = {'limit': 15, 'offset': 5, 'statuses': 'PENDING'}
        _request_mock.assert_called_with(path, params)
        assert_equal(res, [])

    @patch('outbrain.OutbrainAmplifyApi._request')
    @patch('outbrain.OutbrainAmplifyApi.get_token', MagicMock())
    def test_get_campaign(self, _request_mock):
        config = yaml.load(open('outbrain.yml.example', 'r'))
        api = outbrain.OutbrainAmplifyApi(outbrain_config=config)

        path = 'currencies'
        _request_mock.return_value = {}
        result = api.get_currencies()
        _request_mock.assert_called_with(path)
        assert_equal(result, [])

        _request_mock.return_value = {'currencies': 'currency_list_mock'}
        result = api.get_currencies()
        _request_mock.assert_called_with(path)
        assert_equal(result, 'currency_list_mock')


#--------------------------------------------------------------------------------------------------
# Utility methods to ease mocking objects
#--------------------------------------------------------------------------------------------------
def create_mock_token():
    token = MagicMock()
    token.text = '{"OB-TOKEN-V1": {"mock_key": "mock_value"}}'
    return token

def create_mock_request_result():
    result = MagicMock()
    result.text = '{"request_key": "request_value"}'
    return result




from dateutil.parser import parse
from datetime import datetime, timedelta
import requests
import simplejson as json
import yaml

class OutbrainAmplifyApi(object):

    def __init__(self, outbrain_config=None):
        if not outbrain_config:
            outbrain_config = yaml.load(open('outbrain.yml', 'r'))
        self.user = outbrain_config['user']
        self.password = outbrain_config['password']
        self.base_url = outbrain_config['base_url']
        self.token = self.get_token(self.user, self.password)

    def _request(self, path, params={}):
        url = self.base_url + path
        r = requests.get(url, headers={'OB-TOKEN-V1': self.token},
            params=params) 
        return json.loads(r.text)

    def get_token(self, user, password):
        token_url = self.base_url + '/login'
        basic_auth = requests.auth.HTTPBasicAuth(user, password)
        r = requests.get(token_url, auth=basic_auth)
        results = json.loads(r.text)
        return results['OB-TOKEN-V1']

    #----------------------------------------------------------------------------------------------
    # Methods to acquire marketer information
    #----------------------------------------------------------------------------------------------
    def get_marketer(self, marketer_id):
        path = 'marketers/{0}'.format(marketer_id)
        result = self._request(path)
        return result

    def get_marketers(self):
        path = 'marketers'
        results = self._request(path)
        return results.get('marketers', [])

    def get_marketer_ids(self):
        marketers = self.get_marketers()
        return [marketer['id'] for marketer in marketers]

    #----------------------------------------------------------------------------------------------
    # Methods to acquire budget information
    #----------------------------------------------------------------------------------------------
    def get_budget(self, budget_id):
        path = 'budgets/{0}'.format(budget_id)
        result = self._request(path)
        return result

    def get_budgets_per_marketer(self, marketer_ids):
        budgets = {}
        for marketing_id in marketer_ids:
            path = '/marketers/{0}/budgets'.format(marketing_id)
            results = self._request(path)
            marketer_budgets = results.get('budgets', [])
            budgets[marketing_id] = marketer_budgets
        return budgets

    #----------------------------------------------------------------------------------------------
    # Methods to acquire campaign information
    #----------------------------------------------------------------------------------------------
    def get_campaign(self, campaign_id):
        path = 'campaigns/' + campaign_id
        result = self._request(path)
        return result

    def get_campaign_ids(self):
        return [c['id'] for c in self.get_campaigns()]

    def get_campaigns(self):
        return [c for c in self._yield_all_campaigns()]

    def _yield_all_campaigns(self):
        marketer_ids = self.get_marketer_ids()
        marketer_campaigns = self.get_campaigns_per_marketer(marketer_ids)
        for m in marketer_campaigns.keys():
            for c in marketer_campaigns[m]:
                yield c

    def get_campaigns_per_budget(self, budget_ids):
        campaigns = {}
        for budget_id in budget_ids:
            path = 'budgets/{0}/campaigns'.format(budget_id)
            results = self._request(path)            
            budget_campaigns = results.get('campaigns', [])
            campaigns[budget_id] = budget_campaigns
        return campaigns

    def get_campaigns_per_marketer(self, marketing_ids, include_archived=True):
        campaigns = {}
        for marketing_id in marketing_ids:
            path = 'marketers/{0}/campaigns'.format(marketing_id)
            params = {'includeArchived': 'true' if include_archived else 'false'}
            results = self._request(path, params)            
            marketer_campaigns = results.get('campaigns', [])
            campaigns[marketing_id] = marketer_campaigns
        return campaigns

    #----------------------------------------------------------------------------------------------
    # Methods to acquire specific performance information
    #----------------------------------------------------------------------------------------------
    def get_campaign_performace_per_promoted_link(self, campaign_ids, start_day, end_day):
        """
        :returns: dict[campaign_id][publisher_id] = performance_data
        """
        performance = dict()
        for c in campaign_ids:
            path = 'campaigns/{0}/performanceByPromotedLink'.format(c)
            performance[c] = dict()
            result = self._page_performance_data(path, start_day, end_day)
            for data in result:
                performance[c][data['id']] = data
        return performance

    def get_campaign_performace_per_publisher(self, campaign_ids, start_day, end_day):
        """
        :returns: dict[campaign_id][publisher_id] = performance_data
        """
        performance = dict()
        for c in campaign_ids:
            path = 'campaigns/{0}/performanceByPublisher'.format(c)
            performance[c] = dict()
            result = self._page_performance_data(path, start_day, end_day)
            for data in result:
                performance[c][data['id']] = data
        return performance

    def get_marketers_performace_per_section(self, marketer_ids, start_day, end_day):
        """
        :returns: dict[marketer_id][section] = performance_data
        """
        performance = dict()
        for m in marketer_ids:
            path = 'marketers/{0}/performanceBySection'.format(m)
            performance[m] = dict()
            result = self._page_performance_data(path, start_day, end_day)
            for data in result:
                performance[m][data['id']] = data
        return performance

    def get_publisher_performace_per_marketer(self, marketer_ids, start_day, end_day):
        """
        :returns: dict[marketer_id][publisher_id] = performance_data
        """
        performance = dict()
        for m in marketer_ids:
            path = 'marketers/{0}/performanceByPublisher'.format(m)
            performance[m] = dict()
            result = self._page_performance_data(path, start_day, end_day)
            for data in result:
                performance[m][data['id']] = data
        return performance

    def get_campaign_performace_per_section(self, campaign_ids, start_day, end_day):
        """
        :returns: dict[campaign_id][section] = performance_data
        """
        performance = dict()
        for c in campaign_ids:
            path = 'campaigns/{0}/performanceBySection'.format(c)
            performance[c] = dict()
            result = self._page_performance_data(path, start_day, end_day)
            for data in result:
                performance[c][data['id']] = data
        return performance

    #----------------------------------------------------------------------------------------------
    # "Private" helper methods for acquiring/paging performance information
    #----------------------------------------------------------------------------------------------
    def _page_performance_data(self, path, start, end):
        result = []
        offset = 0

        performance = self._get_performance_data(path, start, end, 50, offset)
        while performance:
            result.extend(performance)

            offset += len(performance)
            performance = self._get_performance_data(path, start, end, 50, offset)
        return result

    def _get_performance_data(self, path, start, end, limit, offset):
        params = {'limit': limit,
                  'offset': offset,
                  'from': start.strftime('%Y-%m-%d'),
                  'to': end.strftime('%Y-%m-%d')}
        result = self._request(path, params)
        return result.get('details', [])

    #----------------------------------------------------------------------------------------------
    # Methods to acquire promoted link information
    #----------------------------------------------------------------------------------------------
    def get_promoted_links_per_campaign(self, campaign_ids=[], enabled=None, statuses=[]):
        campaign_ids = campaign_ids or self.get_campaign_ids()
        promoted_links = dict()
        for c in campaign_ids:
            promoted_links[c] = self.get_promoted_links_for_campaign(c, enabled, statuses)
        return promoted_links

    def get_promoted_links_for_campaign(self, campaign_id, enabled=None, statuses=[]):
        return [link for link in self._yield_promoted_links_for_campaign(campaign_id, enabled, statuses)]

    def _yield_promoted_links_for_campaign(self, campaign_id, enabled=None, statuses=[]):
        offset = 0
        path = 'campaigns/{0}/promotedLinks'.format(campaign_id)
        promoted_links = self._page_promoted_links_for_campaign(path, enabled, statuses, 50, offset)
        while promoted_links:
            for pl in promoted_links:
                yield pl

            offset += len(promoted_links)
            promoted_links = self._page_promoted_links_for_campaign(path, enabled, statuses, 50, offset)

    def _page_promoted_links_for_campaign(self, path, enabled, statuses, limit, offset):
        params = {'limit': limit,
                  'offset': offset}

        if enabled is not None:
            params['enabled'] = 'true' if enabled else 'false'
        if statuses:
            params['statuses'] = ','.join(statuses)

        return self._request(path, params).get('promotedLinks', [])

    #----------------------------------------------------------------------------------------------
    # Other methods
    #----------------------------------------------------------------------------------------------
    def get_currencies(self):
        results = self._request('currencies')
        return results.get('currencies', [])



try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup
import os

setup(
    name='outbrain',
    version='0.0.8',
    author='Brian Mason',
    author_email='bmsaon@simplereach.com',
    url='https://github.com/andessen/python-outbrain',
    packages=['outbrain'],
    license='LGPL 2.1 or later',
    description='Wrapper for the Outbrain Amplify API',
    install_requires=[
        'python-dateutil >= 2.1',
        'requests >= 2.4.3',
        'pyyaml >= 3.10',
        'simplejson >= 3.1.0',
    ],
    keywords=['outbrain','api', 'amplify'],
    classifiers=['Intended Audience :: Developers']
)
